/**
 * PETALS - PETALS Services Platform. Copyright (c) 2005 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * ------------------------------------------------------------------------- $Id
 * $ -------------------------------------------------------------------------
 */
package org.ow2.petals.component.framework.util;

import java.util.Set;

import javax.activation.DataHandler;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessageExchangeFactory;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;

import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.message.ExchangeImpl;
import org.w3c.dom.Document;

/**
 * Contains utilities methods for Exchange operations.
 * 
 * @author nsalatge - EBMWebsourcing
 * @since 4.0
 * 
 */
public final class ExchangeUtil {

    /**
     * As all methods of this class are static, no constructor is available.
     */
    private ExchangeUtil() {
        // NOP
    }

    /**
     * copy an exchange
     * 
     * @param source
     * @param target
     * @throws MessagingException
     * @throws PEtALSCDKException
     */
    public static Exchange copyExchange(final MessageExchangeFactory mef, final Exchange source)
            throws MessagingException, PEtALSCDKException {

        final MessageExchange msgEx = mef.createExchange(source.getPattern());
        final Exchange target = new ExchangeImpl(msgEx);

        // Set all values in concrete message exchange to fake message exchange
        if (source.getEndpoint() != null) {
            target.setEndpoint(source.getEndpoint());
        }
        target.setInterfaceName(source.getInterfaceName());
        target.setService(source.getService());
        target.setOperation(source.getOperation());
        target.setStatus(source.getStatus());

        if (source.getError() != null) {
            target.setError(source.getError());
        }

        if (source.getFault() != null) {
            target.setFault(source.getFault());
        }

        if ((source.getInMessage() != null) && (source.getInMessage().getContent() != null)) {
            ExchangeUtil.copyNormalizedMessage(source.getInMessage(), target.getInMessage());
        }

        if ((source.getOutMessage() != null) && (source.getOutMessage().getContent() != null)) {
            ExchangeUtil.copyNormalizedMessage(source.getOutMessage(), target.getOutMessage());
        }

        if (source.getPropertyNames() != null) {
            for (Object property : source.getPropertyNames()) {
                target.setProperty((String) property, source.getProperty((String) property));
            }
        }
        return target;
    }

    /**
     * Copies a message's content, properties and attachments in another.
     * 
     * @param originalNm
     * @param copyNm
     * @throws MessagingException
     *      error while setting attachment
     * @throws PEtALSCDKException
     */
    @SuppressWarnings("unchecked")
    public static void copyNormalizedMessage(final NormalizedMessage originalNm,
            final NormalizedMessage copyNm) throws MessagingException, PEtALSCDKException {
        // setting content, with hardcopy (because if semantic routing...)
        final Document contentDoc = SourceHelper.createDocument(originalNm.getContent());
        copyNm.setContent(SourceHelper.createSource(contentDoc));

        // setting properties
        final Set<String> propertyNames = originalNm.getPropertyNames();
        for (final String propertyName : propertyNames) {
            copyNm.setProperty(propertyName, originalNm.getProperty(propertyName));
        }

        // setting attachments if any
        final Set<String> inAttachmentNames = originalNm.getAttachmentNames();
        if (inAttachmentNames != null) {
            for (final String attachmentName : inAttachmentNames) {
                final DataHandler attDh = originalNm.getAttachment(attachmentName);
                copyNm.addAttachment(attachmentName, attDh);
            }
        }

        // setting security context
        copyNm.setSecuritySubject(originalNm.getSecuritySubject());
    }
}
